#include "pmetrinit.h"
#include "pglobalc.h"
#include "../../GEFEL/xfile.h"
#include "../../METR/SRC/globalc.h"

void pmetr_init (int argc,char *argv)
{
  long i;
  char name[1100];
  XFILE *inm,*intr,*inc;

  //  creation of main objects
  Pcp = new pprobdescc;
  
  //  creation of object of parallel solver for mechanical problem
  Psolm = new psolver (Nproc,Myrank);

  //  creation of object of parallel solver for transport problem
  Psolt = new psolver (Nproc,Myrank);
  

  //  docasne umisteni
  if (Pelq==NULL)    Pelq = new planeelemlq;
  if (Asymlq==NULL)  Asymlq = new axisymlq;
  if (Bar2d==NULL)     Bar2d = new barel2d;    
  //if (Barq2d==NULL)    Barq2d = new barelq2d;    
  

  //  problem description of all coupled problem
  Cp = new probdescc;
  //  problem description of mechanical part of problem
  Mp = new probdesc;
  //  problem description of thermo-hydro part of problem
  Tp = new probdesct;
  //  topology of mechanical part of problem
  Gtm = new gtopology;
  //  topology of thermo-hydro part of problem
  Gtt = new gtopology;
  //  auxiliary united topology (mechanical + thermo-hydro)
  Gtu = new gtopology;
  //  additional topological data about mechanical part of problem
  Mt = new mechtop;
  //  additional topological data about thermo-hydro part of problem
  Tt = new transtop;
  //  additional topological data about thermo-hydro part of problem
  Ct = new couptop;
  //  materials of mechanical part of problem
  Mm = new mechmat;
  //  materials of thermo-hydro part of problem
  Tm = new transmat;
  //  materials of thermo-hydro part of problem
  Cmu = new coupmatu;
  //  materials of thermo-hydro part of problem
  Cml = new coupmatl;
  //  cross-section in mechanical part of problem
  Mc = new mechcrsec;
  //  cross-section in thermo-hydro part of problem
  Tc = new transcrsec;
  //  boundary conditions and load cases in coupled problem
  Cb = new coupbclc;
  //  boundary conditions and load cases in mechanical part of problem
  Mb = new mechbclc;
  //  boundary conditions and load cases in thermo-hydro part of problem
  Tb = new transbclc;
  //  mechanical outdriver
  Outdm = new outdriverm;
  //  transport outdriver
  Outdt = new outdrivert;
  //  coupled problem outdriver
  Outdc = new outdriverc;
  
  //  names of input files
  //strcpy (name,argv[1]);
  strcpy (name,argv);
  //sprintf (&name[strlen (argv[1])],"%d.in",Myrank+1);
  sprintf (&name[strlen (argv)],"%d.in",Myrank+1);
  inc = xfopen (name,"r");
  if (inc==NULL){
    fprintf (stderr,"\n Input file for coupled part is not open on processor %d (file %s, line %d).\n",Myrank,__FILE__,__LINE__);
    abort();
  }
  inc->maxlnsize = 1000;
  inc->warrning = 1;
  inc->kwdmode = ignore;
  //in->kwdmode = sequent_mode;
  inc->ignorecase = 1;
  
  //  number of subdomain
  xfscanf (inc,"%ld",&Ndom);
  Ndom--;
  
  //  problem description of all coupled problem
  Cp->read (inc);
  
  //  reading data describing the problem
  Mp->read (inm);
  
  //  basic problem data
  Tp->read (intr);
  
  //  creation of object of parallel solver
  Psolm = new psolver (Nproc,Myrank);
  
  //  reading data about parallel solver
  Psolt->read (inm,Ndom,Mp->tstorsm,Mespr);
  
  //  creation of additional object
  Psolt = new psolver (Nproc,Myrank);
  
  //  reading data about parallel solver
  Psolt->read (intr,Ndom,Tp->tstorkm,Mesprt);
  
  

  //  input file for mechanical part
  strcpy (name,Cp->minfile);
  sprintf (&name[strlen (Cp->minfile)],"%d.in",Myrank+1);
  inm = xfopen(name,"r");
  if (inm==NULL){
    fprintf (stderr,"\n Input file for mechanical part has not been specified.");
    fprintf (stderr,"\n Try it again!\n\n");
    abort ();
  }
  
  //  input file for thermo-hydro part
  strcpy (name,Cp->tinfile);
  sprintf (&name[strlen (Cp->tinfile)],"%d.in",Myrank+1);
  intr = xfopen(name,"r");
  if (intr==NULL){
    fprintf (stderr,"\n Input file for thermo-hydro part has not been specified.");
    fprintf (stderr,"\n Try it again!\n\n");
    abort ();
  }
  inm->maxlnsize = 1000;
  inm->warrning = 1;
  inm->kwdmode = ignore;
  //in->kwdmode = sequent_mode;
  inm->ignorecase = 1;


  strcpy (name,Cp->minfile);
  sprintf (&name[strlen (Cp->minfile)],"%d.log",Myrank+1);
  Out = fopen (name,"wt");
  if (Out==NULL){
    fprintf (stderr,"\n Log file for MEFEL is not opened on processor %d.\n",Myrank);
  }
  strcpy (name,Cp->tinfile);
  sprintf (&name[strlen (Cp->tinfile)],"%d.log",Myrank+1);
  Outt = fopen (name,"wt");
  if (Outt==NULL){
    fprintf (stderr,"\n Log file for TRFEL is not opened on processor %d.\n",Myrank);
  }
  intr->maxlnsize = 1000;
  intr->warrning = 1;
  intr->kwdmode = ignore;
  //in->kwdmode = sequent_mode;
  intr->ignorecase = 1;
  

  Psolm->procdomcorr ();
  Psolt->procdomcorr ();
  

  //  input of node and element data
  Mt->read (inm);
  Tt->read (intr);
  Ct->read (inc);

  
  if (Cp->tprob==fully_coupled_mech_trans){
    //  unification of two topologies
    Gtu->comptop (Gtm,Gtt);
  }
  
  //  initiation of the mechanical solver
  Psolm->initiate (Gtm);
  //  initiation of the transport solver
  Psolt->initiate (Gtt);
  

  //  reading of the array ltg (local-to-global correspondence)
  Psolm->read_ltg (inm,Gtm,Out);
  Psolt->read_ltg (intr,Gtt,Outt);
  

  //  generation of code numbers for mechanical problem
  Ndofm=Psolm->ordering (Gtm,Out);
  //  generation of code numbers for transport problem
  Ndoft=Psolt->ordering (Gtt,Outt);
  


  if (Cp->tprob==fully_coupled_mech_trans){
    //  generation of code numbers in both parts
    Ndofc=Gtu->gencodnum ();
    //  distribution of code numbers to local topologies
    Gtu->cndistr (Gtm,Gtt);
  }


  //  input of material characteristics
  Mm->read (inm);
  //  definition of variables ncompstr on intergation points
  //  definition of variables ssst on intergation points
  Mm->init_ip_1 ();
  

  Tm->read (intr);

  //int. points allocation and initiation
  Tm->intpointalloc ();
  Tm->intpointinit ();

  
  if (Cp->tprob==fully_coupled_mech_trans){
    Cmu->read (inc);
    Cml->read (inc);
  }
  //  provizorium
  if (Cp->tprob==par_coupl_mech_trans){
    //Cmu->read (inc);
    //Cml->read (inc);
  }
  //  konec provizoria
  
  
  //  input of cross section characteristics
  Mc->read (inm);
  Tc->read (intr);
  

  //  input of loading
  Mb->read (inm);
  Tb->read (intr);
  
  //  searching for elements with nodes where source is defined
  Tb->elemsource ();

  if (Cp->tprob==fully_coupled_mech_trans){  
    //  left and right hand sides
    Lsrsc = new lhsrhsc;
    Lsrsc->alloc ();
    //Lsrs->initcond (in);
    
    
    Lsrs = new lhsrhs;
    Lsrst = new lhsrhst;
    
    Lsrst->lhsi=Lsrsc->lhsi;
    Lsrst->initcond (intr);
  }
  if (Cp->tprob==par_coupl_mech_trans){
    Lsrs = new lhsrhs;
    Lsrs->alloc ();
    Lsrs->initcond (inm);
    
    Lsrst = new lhsrhst;
    Lsrst->alloc ();
    Lsrst->initcond (intr);
    
    
    Mb->alloc_sumcomp ();
    Mm->tempr = new double [Mm->tnip];
    Mm->moist = new double [Mm->tnip];
    for (i=0;i<Mm->tnip;i++){
      Mm->tempr[i]=0.0;
      Mm->moist[i]=0.0;
    }
  }
  

  // input of output description
  //metr
  Outdc->read(inc);
  //mefel
  Outdm->read(inm);
  //trfel
  Outdt->read(intr);
  

  //  names of output files
  strcpy (name,Outdm->outfn);
  sprintf (&name[strlen (Outdm->outfn)],"%d.out",Myrank+1);
  strcpy (Outdm->outfn, name);
  if (Outdm->gf != grfmt_no)
  {
    strcpy (name,Outdm->outgrfn);
    sprintf (&name[strlen (Outdm->outgrfn)],"%d",Myrank+1);
    strcpy (Outdm->outgrfn, name);
  }
  if (Outdm->ndiag > 0)
  {
    strcpy (name,Outdm->outdiagfn);
    sprintf (&name[strlen (Outdm->outdiagfn)],"%d.dat",Myrank+1);
    strcpy (Outdm->outdiagfn, name);
  }  

  //  names of output files
  strcpy (name,Outdt->outfn);
  sprintf (&name[strlen (Outdt->outfn)],"%d.out",Myrank+1);
  strcpy (Outdt->outfn, name);
  if (Outdt->gf != grftt_no)
  {
    strcpy (name,Outdt->outgrfn);
    sprintf (&name[strlen (Outdt->outgrfn)],"%d",Myrank+1);
    strcpy (Outdt->outgrfn, name);
  }
  if (Outdt->ndiag > 0)
  {
    strcpy (name,Outdt->outdiagfn);
    sprintf (&name[strlen (Outdt->outdiagfn)],"%d.dat",Myrank+1);
    strcpy (Outdt->outdiagfn, name);
  }  


  //  zacatek TKo
  //  definition of nonlocal models and models for temperature
  Mm->init_ip_2 ();
  //  konec TKo

  //  allocation of arrays defined on nodes
  Mt->alloc_nodes_arrays ();
  //  number of components in arrays used in outdriver
  //  maximum number of components in array strain/stress on nodes and elements
  Mt->give_maxncompstr(Mm->max_ncompstrn, Mm->max_ncompstre);
  //  maximum number of components in array other on nodes and elements
  Mt->give_maxncompo(Mm->max_nncompo, Mm->max_encompo);
  //  konec TKo


  //  prescribed displacements on elements investigation
  Mt->elemprescdisp ();
  //  reactions on elements investigation
  if (Mp->reactcomp==1){
    Mt->comreac ();
  }

  Mm->alloceigstrains ();
  Mm->alloctempstrains ();
  Mm->alloceigstresses ();
  
  //  nodes allocation
  Tt->alloc_nodes ();
  Mt->alloc_nodes_arrays ();
  
  
  Mp->temperature=1;
  
  xfclose (inc);
  xfclose (inm);
  xfclose (intr);
  
  Pcp->shift_indices ();
  
  strcpy (name,argv);
  //sprintf (&name[strlen (argv[1])],"%d.in",Myrank+1);
  sprintf (&name[strlen (argv)],"%d.log",Myrank+1);
  Outc = fopen (name,"w");
 

}
